//
//  ASPointControlPluginView.m
//  ASPointControlPlugin
//
//  Created by 佐藤昭 on 11/02/05.
//  Copyright 2011 SatoAkira. All rights reserved.
//

#import "ASPointControl.h"
#import "ASPointControlCell.h"

static Class _myStoredCellClass = nil;

@interface ASPointControl () <NSCoding>

@property	BOOL acceptsAllowKeys;	// 矢印キーの使用を許すか否か。デフォルトはYES。 //
@property	NSString *arrowKeysString;	// right,up,left,downの順 //
@property	NSString *observedKeyPathForPoint; // for binding //
@property	id observedObjectForPoint; // for binding //
@property	NSCursor *theCursor;
@property	NSPoint mousePoint; // 画面の座標系とは全く無関係 //

@end

@implementation ASPointControl

/* @synthesize mousePoint; */

NSString *ASPointControlMousePointKey = @"mousePoint";

+ (void)initialize {
	if (self == [ASPointControl class])
		[self setCellClass:[ASPointControlCell class]];
}
+ (Class)cellClass {
	return _myStoredCellClass;
}

+ (void)setCellClass:(Class)classId {
	_myStoredCellClass = classId;
	
}
- (void)resetCursorRects {
	NSRect visible = self.visibleRect; // イメージ全体の大きさが返される。 //
	
	if( NO == NSIsEmptyRect(visible) ) {
		[self addCursorRect:visible cursor:self.theCursor];
#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_12
		self.theCursor.onMouseEntered = YES; // YES if the receiver accepts future mouseEntered: event messages; otherwise it ignores them. //
#endif
	}
}
- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent {
	// 最初のマウスダウンがあったときに一回だけ呼ばれる //
	return [self.window makeFirstResponder:self];
}
- (BOOL)resignFirstResponder {
	// 最初にファーストレスポンダでなくなるときに一回だけ呼ばれるが、[self setNeedsDisplay:YES]はその都度実行される。 //
	[self updateCell:self.cell];
	return YES;
}
- (instancetype)initWithFrame:(NSRect)frameRect {
	// IBから作られた場合には、ここは呼ばれないのでmousePointを初期化しておかねばならない。 //
    BOOL error = (nil == (self = [super initWithFrame:frameRect]));
	
	if (NO == error) {
		Class cellClass = self.class.cellClass;
		ASPointControlCell *myCell = [[cellClass alloc] initImageCell:nil];
		NSPoint minPoint = [[myCell valueForKey:@"minPoint"] pointValue];
		NSPoint maxPoint = [[myCell valueForKey:@"maxPoint"] pointValue];
		unichar arrows[] = {0xf703,0xf700,0xf702,0xf701}; // 矢印キーは /Uf700~/Uf703 //
		
		self.cell = myCell;
		self.mousePoint = NSMakePoint(0.5 * (minPoint.x + maxPoint.x),0.5 * (minPoint.y + maxPoint.y));
		[myCell release];
		[self.theCursor release];
		self.theCursor = [[NSCursor crosshairCursor] retain];
		self.acceptsAllowKeys = YES;
		self.arrowKeysString = [[NSString alloc] initWithCharacters:arrows length:4];
    }
    return error ? nil : self;
}
- (instancetype)init {
	return [self initWithFrame:NSMakeRect(0.0,0.0,64.0,64.0)];
}
- (void)dealloc {
	[self.observedObjectForPoint release];
	[self.observedKeyPathForPoint release];
	[self.theCursor release];
	[self.arrowKeysString release];
	[super dealloc];
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
	// IBから作られた場合には、ここだけで初期化する。 //
	if (nil != (self = [super initWithCoder:aDecoder])) {
		Class cellClass = self.class.cellClass;
		ASPointControlCell *myCell = [[cellClass alloc] initImageCell:nil];
		NSPoint minPoint = [[myCell valueForKey:@"minPoint"] pointValue];
		NSPoint maxPoint = [[myCell valueForKey:@"maxPoint"] pointValue];
		unichar arrows[] = {0xf703,0xf700,0xf702,0xf701}; // 矢印キーは /Uf700~/Uf703 //
		
		self.cell = myCell;
		self.mousePoint = NSMakePoint(0.5 * (minPoint.x + maxPoint.x),0.5 * (minPoint.y + maxPoint.y));
		[myCell release];
		self.theCursor = [[NSCursor crosshairCursor] retain];
		self.acceptsAllowKeys = YES;
		self.arrowKeysString = [[NSString alloc] initWithCharacters:arrows length:4];
		self.observedObjectForPoint = [[aDecoder decodeObjectForKey:@"observedObjectForPointKey"] retain];
		self.observedKeyPathForPoint = [[aDecoder decodeObjectForKey:@"observedKeyPathForPointKey"] retain];
	}
	return self; // このあとinitWithFrame:が呼ばれるとは限らない。 //
}
- (void)encodeWithCoder:(NSCoder *)aCoder {
	[super encodeWithCoder:aCoder];
	[aCoder encodeObject:self.theCursor forKey:@"theCursorKey"];
	[aCoder encodePoint:self.mousePoint forKey:@"mousePointKey"];
	[aCoder encodeBool:self.acceptsAllowKeys forKey:@"acceptsAllowKeys"];
	[aCoder encodeObject:self.arrowKeysString forKey:@"arrowKeysString"];
	[aCoder encodeObject:self.observedObjectForPoint forKey:@"observedObjectForPointKey"];
	[aCoder encodeObject:self.observedKeyPathForPoint forKey:@"observedKeyPathForPointKey"];
}
- (void)keyDown:(NSEvent *)theEvent {
	if (YES == self.acceptsAllowKeys) {
		if ((0 == [self.arrowKeysString rangeOfString:theEvent.charactersIgnoringModifiers].length) || (NO == self.isEditable) || (NO == self.isEnabled))
			[super keyDown:theEvent];
	}
	else
		[super keyDown:theEvent];
}
- (void)keyUp:(NSEvent *)theEvent {
	if ((YES == self.acceptsAllowKeys) && (YES == self.isEditable) && (YES == self.isEnabled)) {
		NSString *aString = theEvent.charactersIgnoringModifiers;
		NSRange arrowRange = [self.arrowKeysString rangeOfString:aString];
		if (0 < arrowRange.length) {
			NSPoint minPoint = ((ASPointControlCell *)self.cell).minPoint;
			NSPoint maxPoint = ((ASPointControlCell *)self.cell).maxPoint;
			NSPoint midPoint = NSMakePoint(0.5 * (minPoint.x + maxPoint.x), 0.5 * (minPoint.y + maxPoint.y));
			NSPoint newPoint = self.mousePoint;
			switch (arrowRange.location) {
				case 0:
					newPoint.x = (YES == self.horizontalFlipped) ? (self.mousePoint.x > midPoint.x) ? midPoint.x : minPoint.x  :  (self.mousePoint.x >= midPoint.x) ? maxPoint.x : midPoint.x;
					break;
				case 1:
					newPoint.y = (YES == self.verticalFlipped) ? (self.mousePoint.y > midPoint.y) ? midPoint.y : minPoint.y  :  (self.mousePoint.y >= midPoint.y) ? maxPoint.y : midPoint.y;
					break;
				case 2:
					newPoint.x = (YES == self.horizontalFlipped) ? (self.mousePoint.x >= midPoint.x) ? maxPoint.x : midPoint.x  :  (self.mousePoint.x > midPoint.x) ? midPoint.x : minPoint.x;
					break;
				case 3:
					newPoint.y = (YES == self.verticalFlipped) ? (self.mousePoint.y >= midPoint.y) ? maxPoint.y : midPoint.y  :  (self.mousePoint.y > midPoint.y) ? midPoint.y : minPoint.y;
					break;
				default:
					break;
			}
			self.mousePoint = newPoint; // KVO //
			[self writeToOutlet];
			if (NO == self.isContinuous)
				[self drawCellInside:self.cell];				
		}
		else
			[super keyUp:theEvent];
	}
	else
		[super keyUp:theEvent];
}
- (void)setBordered:(BOOL)flag {
	self.cell.bordered = flag;
}
- (BOOL)isBordered {
	return self.cell.isBordered;
}
- (void)setEnabled:(BOOL)flag {
	self.cell.enabled = flag;
}
- (BOOL)isEnabled {
	return self.cell.isEnabled;
}
- (void)setEditable:(BOOL)flag {
	self.cell.editable = flag;
}
- (BOOL)isEditable {
	return self.cell.isEditable;
}
- (void)setDrawsTickMark:(BOOL)flag {
	((ASPointControlCell *)self.cell).drawsTickMark = flag;
}
- (BOOL)drawsTickMark {
	return ((ASPointControlCell *)self.cell).drawsTickMark;
}
- (void)setImage:(NSImage *)anImage {
	self.cell.image = anImage;
}
- (NSImage *)image {
	return self.cell.image;
}
- (void)setHorizontalFlipped:(BOOL)flag {
	((ASPointControlCell *)self.cell).horizontalFlipped = flag;
}
- (BOOL)horizontalFlipped {
	return ((ASPointControlCell *)self.cell).horizontalFlipped;
}
- (void)setVerticalFlipped:(BOOL)flag {
	((ASPointControlCell *)self.cell).verticalFlipped = flag;
}
- (BOOL)verticalFlipped {
	return ((ASPointControlCell *)self.cell).verticalFlipped;
}
- (void)setSquare:(BOOL)flag {
	((ASPointControlCell *)self.cell).square = flag;
}
- (BOOL)square {
	return ((ASPointControlCell *)self.cell).square;
}
- (void)setMinPoint:(NSPoint)aPoint {
	((ASPointControlCell *)self.cell).minPoint = aPoint;
}
- (void)setMaxPoint:(NSPoint)aPoint {
	((ASPointControlCell *)self.cell).maxPoint = aPoint;
}
- (void)writeToOutlet {
	if (nil != self.observedObjectForPoint)
		[self.observedObjectForPoint setValue:[NSValue valueWithPoint:self.mousePoint] forKeyPath:self.observedKeyPathForPoint];
	if (YES == self.isContinuous)
		[self drawCellInside:self.cell];
}

#pragma mark Binding

static void *PointControlBindContext = (void *)@"PointControl";

- (void)bind:(NSString *)binding toObject:(id)observableObject withKeyPath:(NSString *)keyPath options:(NSDictionary *)options {
	if (YES == [binding isEqualToString:ASPointControlMousePointKey]) {
		if (![[[NSBundle mainBundle] bundleIdentifier] isEqualToString:@"com.apple.InterfaceBuilder3"]) {
			[observableObject addObserver:self forKeyPath:keyPath options:0 context:PointControlBindContext];
			self.observedObjectForPoint = [observableObject retain];
			self.observedKeyPathForPoint = [keyPath copy];
		}
	}
	[super bind:binding toObject:observableObject withKeyPath:keyPath options:options];
}
- (void)unbind:(NSString *)bindingName {
	if (YES == [bindingName isEqualToString:ASPointControlMousePointKey]) {
		if (![[[NSBundle mainBundle] bundleIdentifier] isEqualToString:@"com.apple.InterfaceBuilder3"]) {
			[self.observedObjectForPoint release];
			self.observedObjectForPoint = nil;
			[self.observedKeyPathForPoint release];
			self.observedKeyPathForPoint = nil;
		}
	}
	[super unbind:bindingName];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	if (context == PointControlBindContext) {
		id newPoint = [self.observedObjectForPoint valueForKeyPath:self.observedKeyPathForPoint];
		if ((nil != newPoint) && (YES == [newPoint isKindOfClass:[NSValue class]])) {
			self.mousePoint = ((NSValue *)newPoint).pointValue;
			[self drawCellInside:self.cell];
		}
		// newPointがNSStateMarkerクラスになることが頻繁にある。 //
	}
	else
		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
}

@end
